from django.test import TransactionTestCase
from channels.routing import URLRouter
from django.urls import reverse

from whitebox.routing import websocket_urlpatterns
from plugin.manager import plugin_manager


class TestWhiteboxPluginASDBGPSIntegration(TransactionTestCase):
    def setUp(self):
        plugin_manager.plugins = []
        plugin_manager.previously_discovered_plugins = {}
        plugin_manager.discover_plugins()

        self.plugin = next(
            (
                x
                for x in plugin_manager.plugins
                if x.__class__.__name__ == "WhiteboxPluginASDBGPS"
            ),
            None,
        )
        self.application = URLRouter(websocket_urlpatterns)

        return super().setUp()

    def test_plugin_loaded(self):
        self.assertIsNotNone(self.plugin)

    def test_plugin_template_and_static_files(self):
        response = self.client.get(reverse("index"))
        self.assertEqual(response.status_code, 200)
        self.assertIn(
            "whitebox_plugin_adsb_gps/whitebox_plugin_adsb_gps.html",
            response.context["templates"],
        )
        self.assertIn(
            "/static/whitebox_plugin_adsb_gps/whitebox_plugin_adsb_gps.js",
            response.context["js_files"],
        )
